@@ -8,6 +8,7 @@ use DBIx::SearchBuilder::Handle;
 @ISA = qw (DBIx::SearchBuilder::Record);
 
 my %_RECORD_CACHE = (); 
+my %_KEY_CACHE = (); 
 
 
 # Function: new 
@@ -42,6 +43,38 @@ sub _RecordCache {
     return(\%_RECORD_CACHE);
 }
 
+# Function: _KeyCache
+# Type    : private instance
+# Args    : none
+# Lvalue  : hash: KeyCache
+# Desc    : Returns a reference to the Key cache hash
+
+sub _KeyCache {
+    my $this = shift;
+    return(\%_KEY_CACHE);
+}
+
+
+
+# Function: LoadFromHash
+# Type    : (overloaded) public instance
+# Args    : See DBIx::SearchBuilder::Record::LoadFromHash
+# Lvalue  : array(boolean, message)
+
+sub LoadFromHash {
+    my $this = shift;
+    my ($rvalue, $msg) = $this->SUPER::LoadFromHash(@_);
+
+    my $cache_key = $this->_gen_primary_cache_key();
+
+    ## Check the return value, if its good, cache it! 
+    if ($rvalue) {
+     ## Only cache the object if its okay to do so. 
+        $this->_store() if ($this->{'_CacheConfig'}{'cache_p'});
+    }
+
+    return($rvalue,$msg);
+}
 
 # Function: LoadByCols
 # Type    : (overloaded) public instance
@@ -52,15 +85,17 @@ sub LoadByCols {
   my ($this, %attr) = @_; 
 
   ## Generate the cache key
-  my $cache_key=$this->_gen_cache_key(%attr);
+  my $alternate_key=$this->_gen_cache_key(%attr);
+  my $cache_key = $this->_lookup_primary_cache_key($alternate_key);
 
-  if (exists $this->_RecordCache->{$cache_key}) { 
+
+  if ($cache_key && exists $this->_RecordCache->{$cache_key}) { 
     $cache_time = $this->_RecordCache->{$cache_key}{'time'};
 
     ## Decide if the cache object is too old
     if ((time() - $cache_time) <= $this->{'_CacheConfig'}{'cache_for_sec'}) {
-	$this->_fetch($cache_key); 
-	return (1, "Fetched from cache");
+	    $this->_fetch($cache_key); 
+	    return (1, "Fetched from cache");
     }
     else { 
       $this->_gc_expired();
@@ -73,19 +108,18 @@ sub LoadByCols {
   ## Check the return value, if its good, cache it! 
   if ($rvalue) {
     ## Only cache the object if its okay to do so. 
-    $this->_store($cache_key) if ($this->{'_CacheConfig'}{'cache_p'});
-    return ($rvalue, $msg);
-  }
-  else { 
-    return ($rvalue, $msg);
+    $this->_store() if ($this->{'_CacheConfig'}{'cache_p'});
+
+    my $new_cache_key = $this->_gen_primary_cache_key();
+    $this->_KeyCache->{$alternate_key} = $new_cache_key;
+    $this->_KeyCache->{$alternate_key}{'time'} = time();
   }
 
-  return (0, "Unexpected something or other [never hapens].");
+  return ($rvalue, $msg);
+
 }
 
 
-
-
 # Function: _Set
 # Type    : (overloaded) public instance
 # Args    : see DBIx::SearchBuilder::Record::_Set
@@ -93,7 +127,7 @@ sub LoadByCols {
 
 sub _Set () { 
   my ($this, %attr) = @_; 
-  my $cache_key = $this->{'_CacheConfig'}{'cache_key'};
+  my $cache_key = $this->_gen_primary_cache_key();
 
   if (exists $this->_RecordCache->{$cache_key}) {
     $this->_expire($cache_key);
@@ -104,6 +138,23 @@ sub _Set () {
 }
 
 
+# Function: Delete
+# Type    : (overloaded) public instance
+# Args    : nil
+# Lvalue  : ?
+
+sub Delete () { 
+  my ($this) = @_; 
+  my $cache_key = $this->_gen_primary_cache_key();
+
+  $this->_expire($cache_key);
+ 
+  return $this->SUPER::Delete();
+
+}
+
+
+
 
 
 # Function: _gc_expired
@@ -115,6 +166,11 @@ sub _Set () {
 sub _gc_expired () { 
   my ($this) = @_; 
   
+  foreach $cache_key (keys %{$this->_KeyCache}) {
+    my $cache_time = $this->_RecordCache->{$cache_key}{'time'};  
+    $this->_expire($cache_key) 
+      if ((time() - $cache_time) > $this->{'_CacheConfig'}{'cache_for_sec'});
+  }
   foreach $cache_key (keys %{$this->_RecordCache}) {
     my $cache_time = $this->_RecordCache->{$cache_key}{'time'};  
     $this->_expire($cache_key) 
@@ -164,7 +220,8 @@ sub _fetch () {
 # Desc    : Stores this object in the cache. 
 
 sub _store (\$) { 
-  my ($this, $cache_key) = @_; 
+  my ($this) = @_; 
+  my $cache_key = $this->_gen_primary_cache_key();
   $this->{'_CacheConfig'}{'cache_key'} = $cache_key;
   $this->_RecordCache->{$cache_key}{'obj'}=$this;
   $this->_RecordCache->{$cache_key}{'time'}=time();
@@ -192,6 +249,59 @@ sub _gen_cache_key {
 }
 
 
+# Function: _fetch_cache_key
+# Type    : private instance
+# Args    : nil
+# Lvalue  : 1
+
+sub _fetch_cache_key {
+    my ($this) = @_;
+    my $cache_key = $this->{'_CacheConfig'}{'cache_key'};
+    return($cache_key);
+}
+
+
+
+# Function: _gen_primary_cache_key 
+# Type    : private instance
+# Args    : none
+# Lvalue: : 1
+# Desc    : generate a primary-key based variant of this object's cache key
+#           primary keys is in the cache 
+
+sub _gen_primary_cache_key {
+    my ($this) = @_;
+
+    my $primary_cache_key = $this->Table() . ':';
+    foreach my $key (@{$this->_PrimaryKeys}) {
+        $primary_cache_key .= $key.'='.$this->__Value($key).',';
+    }
+    chop ($primary_cache_key);
+    return($primary_cache_key);
+
+}
+
+
+# Function: lookup_primary_cache_key 
+# Type    : private class
+# Args    : string(alternate cache id)
+# Lvalue  : string(cache id)
+sub _lookup_primary_cache_key {
+    my $this = shift;
+    my $alternate_key = shift;
+    if (exists $this->_KeyCache->{$alternate_key}) { 
+         $cache_time = $this->_KeyCache->{$alternate_key}{'time'};
+
+    ## Decide if the cache object is too old
+    if ((time() - $cache_time) <= $this->{'_CacheConfig'}{'cache_for_sec'}) {
+         return $this->_KeyCache->{$alternate_key};
+    }
+    else { 
+      $this->_gc_expired();
+    }
+  } 
+  return (undef)
+}
 
 
 package __CachableDefaults; 
